/*   
    copyright: Steve Dekorte, 2004. All rights reserved.
    license: See _BSDLicense.txt.

    object: SkipDBRecord
    module: SkipDB
    description: 
*/

#ifndef SkipDBRecord_DEFINED
#define SkipDBRecord_DEFINED 1

#include "List.h"
#include "BStream.h"
#include "UDB.h"

typedef struct SkipDBRecord SkipDBRecord;

struct SkipDBRecord
{
    int level;
    uint32_t *pids;
    SkipDBRecord **records;
    
    ByteArray *key;
    uint32_t valuePid;

    void *db;
    PID_TYPE pid;
    unsigned int retainCount;
    int needsSave;
};

SkipDBRecord *SkipDBRecord_new(void);
SkipDBRecord *SkipDBRecord_newWithDB_(void *db);
void SkipDBRecord_free(SkipDBRecord *self);

SkipDBRecord *SkipDBRecord_retain(SkipDBRecord *self);
void SkipDBRecord_release(SkipDBRecord *self);

void SkipDBRecord_db_(SkipDBRecord *self, void *db);

void SkipDBRecord_pid_(SkipDBRecord *self, PID_TYPE pid);
PID_TYPE SkipDBRecord_pid(SkipDBRecord *self);


int SkipDBRecord_level(SkipDBRecord *self);
void SkipDBRecord_level_(SkipDBRecord *self, int level);
void SkipDBRecord_atLevel_setPid_(SkipDBRecord *self, int level, PID_TYPE pid);
PID_TYPE SkipDBRecord_pidAtLevel_(SkipDBRecord *self, int level);
void SkipDBRecord_copyLevel_from_(SkipDBRecord *self, int level, SkipDBRecord *other);

SkipDBRecord *SkipDBRecord_recordAtLevel_(SkipDBRecord *self, int level);

/* --------------------------------------- */

void SkipDBRecord_keyDatum_(SkipDBRecord *self, UDBDatum k);
ByteArray *SkipDBRecord_key(SkipDBRecord *self);

void SkipDBRecord_valueDatum_(SkipDBRecord *self, UDBDatum v);
UDBDatum SkipDBRecord_valueDatum(SkipDBRecord *self);

/* --- serialization ------------------------------------ */

void SkipDBRecord_toStream_(SkipDBRecord *self, BStream *s);
void SkipDBRecord_fromStream_(SkipDBRecord *self, BStream *s);

/* --- search ------------------------------------ */

SkipDBRecord *SkipDBRecord_find_(SkipDBRecord *self, UDBDatum key);
void SkipDBRecord_willRemove_(SkipDBRecord *self, SkipDBRecord *other);

void SkipDBRecord_show(SkipDBRecord *self);

#endif
